%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Project name: 
%
% Simulation of a column pulse tracer test
% PHREEQC function
%
% PHREEQC datablock: SOLUTION, REACTION, EQUILIBRIUM_PHASES and TRANSPORT 
% 
% Code written by Dominique Claveau-Mallet
% Creation: 2019-04-03
% Updated: 2019-04-04
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Description of input data
%
% hydraulic properties (to be changed until proper calibration)
% parm.D: dispersivity (cm)
% parm.n: total porosity (-)
% parm.ne: effective porosity (-)
% parm.Dn: exchange factor between effective and immobile porosity (s-1)
% parm.factor: correction factor between 0 and 1 to reduce the injected mass (in case the tail is not well reproduced)
%
% numerical
% parm.database_path: full path of the database used in PHREEQC simulation
%    example: 'C:\Program Files (x86)\USGS\Phreeqc Interactive 3.3.8-11728\database\phreeqc.dat'
% parm.nbcell: number of numerical cells

% slag properties
% parm.slag.rho: slag density (g/mL)
%
% reactor geometry (1D column)
% parm.lenght: column lenght (m)
% parm.radius: column radius (m)
%
% feeding
% parm.duration: duration of the simulation (d)
% parm.HRTVe: effective hydraulic retention time of voids (h)
% parm.mass: injected tracer mass (ug)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Description of output data
%
% F: raw PHREEQC output file
% output: composition of effluent. matrix with columns 
% time (d) and tracer concentration (ug/L)
%             
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [output,F] = PHREEQCfct_tracer(parm)

phreeqc = actxserver('IPhreeqcCOM.Object');
phreeqc.LoadDatabase(sprintf('%s',parm.database_path)); % retrieve database
phreeqc.ClearAccumulatedLines();

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PHREEQC Code

% Calculations - basic parameters

phreeqc.AccumulateLine('CALCULATE_VALUES')
phreeqc.AccumulateLine('Time_d')
phreeqc.AccumulateLine('-start')
phreeqc.AccumulateLine('10	a = TOTAL_TIME/(60*60*24)')
phreeqc.AccumulateLine('20	SAVE a')
phreeqc.AccumulateLine('-end')
phreeqc.AccumulateLine('Na_ug_L')
phreeqc.AccumulateLine('-start')
phreeqc.AccumulateLine('10	a = TOT("Na")*1000000*23')
phreeqc.AccumulateLine('20	SAVE a')
phreeqc.AccumulateLine('-end')

% Data extraction

phreeqc.AccumulateLine('SELECTED_OUTPUT')
phreeqc.AccumulateLine('-reset	false')
phreeqc.AccumulateLine('-sim')
phreeqc.AccumulateLine('-calculate_values	Time_d Na_ug_L')

% Preparation of the tracer solution

concinj = (1e-9)*parm.factor*parm.mass/(23*parm.length*pi()*(parm.radius^2)*parm.ne);
phreeqc.AccumulateLine('SOLUTION 0')
phreeqc.AccumulateLine('REACTION 0')
phreeqc.AccumulateLine(sprintf('NaCl %.16d', concinj))
phreeqc.AccumulateLine('Save solution 0')
phreeqc.AccumulateLine('end')

% Water in the column at time zero

phreeqc.AccumulateLine(sprintf('SOLUTION 1-%i',2*parm.nbcell+1))
phreeqc.AccumulateLine('pH 7')
phreeqc.AccumulateLine('end')

% Pulse injection of tracer

phreeqc.AccumulateLine('TRANSPORT 1')
phreeqc.AccumulateLine(sprintf('-cells %i',parm.nbcell))
phreeqc.AccumulateLine(sprintf('-lengths %g',parm.length))
phreeqc.AccumulateLine('-shifts 1')
phreeqc.AccumulateLine('-time_step 1 sec')
phreeqc.AccumulateLine(sprintf('-dispersivities %.16d',parm.D))
phreeqc.AccumulateLine(sprintf('-punch_cells %i',parm.nbcell))
phreeqc.AccumulateLine(sprintf('-stagnant	1  %.16d %.16d %.16d',parm.Dn,parm.ne,parm.n-parm.ne))
phreeqc.AccumulateLine('end')

% Feeding with distilled water after the pulse injection

phreeqc.AccumulateLine('SOLUTION 0') %new influent
phreeqc.AccumulateLine('pH 7')
phreeqc.AccumulateLine('Save solution 0')
phreeqc.AccumulateLine('end')
phreeqc.AccumulateLine('TRANSPORT 2')
phreeqc.AccumulateLine(sprintf('-shifts %.16g',24*parm.duration*parm.nbcell/parm.HRTVe))
phreeqc.AccumulateLine(sprintf('-time_step %.16d hour',parm.HRTVe/parm.nbcell))
phreeqc.AccumulateLine('end')
phreeqc.RunAccumulated();

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data extraction

F=phreeqc.GetSelectedOutputArray();
for i=8:length(F(:,1))
output(i-7,:)=[F{i,2},F{i,3}];
end
